# Varnish as a Kubernetes Ingress

Varnish as an Ingress is realized by loading a
[VCL](https://varnish-cache.org/docs/trunk/reference/vcl.html) configuration
that:

* defines
  [directors](https://varnish-cache.org/docs/trunk/users-guide/vcl-backends.html#directors)
  that correspond to Services mentioned in the Ingress definition

  * This is currently hard-wired as the round-robin director.

* defines
  [backends](https://varnish-cache.org/docs/trunk/users-guide/vcl-backends.html)
  corresponding to the Endpoints of the Services. These are assigned to the
  director defined for the Service.

  * Endpoint definitions for a Service are obtained from an API query
    when the VCL configuration is generated, so that the assignments
    of Endpoints is current at VCL generation time.

  * The Varnish backend definitions currently only include the Endpoints'
    IP addresses and ports. There is presently no means to define other
    features of a backend, such as health checks and timeouts.

* generates VCL code to implement the routing of requests to backends
  based on the Host header and/or URL path according to the IngressRules
  given in the Ingress definition.

  * Varnish may cache responses according to its default rules for
    caching, and of course cache hits are delivered without routing the
    requests further.

  * In case of a non-cache-hit (miss or pass), the Host header and/or
    URL path is evaluated according to the IngressRules, and matching
    requests are assigned to the director corresponding to the matched
    Service.

      * An IngressSpec is rejected if it does not specify any Host header.

      * TLS configuration in the IngressSpec is currently ignored.

  * The director in turn chooses a backend corresponding to an Endpoint
    according to its load balancing algorithm (currently only round-robin).

  * If the request does not match any Service according to the
    IngressRules, then:

      * If a default Backend (a Service) was defined in the IngressSpec,
        then the request is assigned to the corresponding director.

      * Otherwise, a synthetic 404 Not Found response is generated by
        Varnish.

  * If there is no valid Ingress definition (none has been defined
    since the Varnish instance started, or the only valid definition
    was deleted), then Varnish generates a synthetic 503 Service Not
    Available response for every request.
